package com.mortals.xhx.module.skin.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.xhx.module.skin.model.*;
import com.mortals.xhx.module.skin.service.SkinFieldService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.skin.dao.SkinBaseDao;
import com.mortals.xhx.module.skin.service.SkinBaseService;
import org.springframework.util.ObjectUtils;

import java.util.List;

/**
 * SkinBaseService
 * 系统基础皮肤 service实现
 *
 * @author zxfei
 * @date 2022-05-24
 */
@Service("skinBaseService")
public class SkinBaseServiceImpl extends AbstractCRUDServiceImpl<SkinBaseDao, SkinBaseEntity, Long> implements SkinBaseService {

    @Autowired
    private SkinFieldService skinFieldService;

    @Override
    protected void validData(SkinBaseEntity entity, Context context) throws AppException {
        SkinBaseQuery skinBaseQuery = new SkinBaseQuery();
        skinBaseQuery.setName(entity.getName());
        SkinBaseEntity skinBaseEntity = this.selectOne(skinBaseQuery);
        if(!ObjectUtils.isEmpty(skinBaseEntity)&&entity.newEntity()){
            throw new AppException("皮肤名称已存在");
        }
        super.validData(entity, context);
    }

    @Override
    protected void findAfter(SkinBaseEntity params, Context context, List<SkinBaseEntity> list) throws AppException {
        list.stream().forEach(item -> {
            List<SkinFieldEntity> skinFieldEntities = skinFieldService.find(new SkinFieldQuery().skinId(item.getId()));
            item.setSkinFieldList(skinFieldEntities);
        });
        super.findAfter(params, context, list);
    }


    @Override
    protected void saveAfter(SkinBaseEntity entity, Context context) throws AppException {
        //删除关联后新增
        saveSkinFields(entity, context);
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(SkinBaseEntity entity, Context context) throws AppException {
        saveSkinFields(entity, context);
        super.updateBefore(entity, context);
    }

    private void saveSkinFields(SkinBaseEntity entity, Context context) {
        Long[] ids = skinFieldService.find(new SkinFieldQuery().skinId(entity.getId())).stream().map(item -> item.getId()).toArray(Long[]::new);
        if (!ObjectUtils.isEmpty(ids)) {
            skinFieldService.remove(ids, context);
        }
        if (!ObjectUtils.isEmpty(entity.getSkinFieldList())) {
            entity.getSkinFieldList().stream().forEach(item->{
                item.setSkinId(entity.getId());
            });
            skinFieldService.save(entity.getSkinFieldList(), context);
        }
    }

    @Override
    protected void saveBefore(SkinBaseEntity entity, Context context) throws AppException {
        //todo 合成实现css模板文件

        super.saveBefore(entity, context);
    }


    private void genTemplateCss(SkinBaseEntity entity, Context context) {
      //加载模板，合成模板数据


        //更新时候删除


    }
}